 /*
 * $Id: edsam.h,v 1.14 2005/09/01 08:27:46 lindahl Exp $
 * 
 *                This source code is part of
 * 
 *                 G   R   O   M   A   C   S
 * 
 *          GROningen MAchine for Chemical Simulations
 * 
 *                        VERSION 3.2.0
 * Written by David van der Spoel, Erik Lindahl, Berk Hess, and others.
 * Copyright (c) 1991-2000, University of Groningen, The Netherlands.
 * Copyright (c) 2001-2004, The GROMACS development team,
 * check out http://www.gromacs.org for more information.

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * If you want to redistribute modifications, please consider that
 * scientific software is very special. Version control is crucial -
 * bugs must be traceable. We will be happy to consider code for
 * inclusion in the official distribution, but derived work must not
 * be called official GROMACS. Details are found in the README & COPYING
 * files - if they are missing, get the official version at www.gromacs.org.
 * 
 * To help us fund GROMACS development, we humbly ask that you cite
 * the papers on the package - you can find them in the top README file.
 * 
 * For more info, check our website at http://www.gromacs.org
 * 
 * And Hey:
 * Gromacs Runs On Most of All Computer Systems
 */

#ifndef _edsam_h
#define _edsam_h

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "types/edsams.h"
#include "vec.h"


/**
 ** debug helpers
 **/
void dump_rotmat(FILE*,matrix rotmat);

void dump_mat(FILE*,int dim,double **mat);

void dump_rvec(FILE *out,int dim,rvec *x);

void dump_edi(FILE*,t_edpar *edi);






/**
 ** flooding
 **/
void init_flood(t_edpar *edi,real dt);
/* called by init_edi, configure some flooding related variables and structures, print headers to output files */

extern void do_flood(FILE *log,t_commrec *cr,rvec x[],rvec force[],t_edsamyn *edyn,int step);
/* main hook - called from do_force() in mdrun*/

void write_edo_flood(t_edpar *edi,int step);

void flood_project(rvec *x,t_edpar *edi);
/* project fitted structure onto supbspace -> store in edi->vec.flood.xproj */

real flood_energy(t_edpar *edi);
/* from flood.xproj compute the Vfl(x) at this point -> store in edi->flood.Vfl */

void flood_forces(t_edpar *edi);
/* from the position and the Vfl compute forces in subspace -> stored in edi->vec.flood.fproj */

void flood_blowup(t_edpar *edi,rvec *forces_cart);
/* raise forces from subspace into cartesian space */

void update_adaption(t_edpar *edi);
/* update the values of Efl, deltaF depending on tau and Vfl */

void do_single_flood(FILE *log,rvec x_orig[],rvec force[],t_edpar *edi,int step,int nr);

void get_flood_enx_names(t_edpar *edi,char**,int *nnames); 
/* get header of energies */

void get_flood_energies(t_edpar *edi,real Vfl[],int nnames); 
/* fl has to be big enough to capture nnames-many entries */

void free_t_edflood(t_edflood *p);







/**
 ** essential dynamics 
 **/ 
extern void prepare_edsam(int step,int start,int homenr,t_commrec *cr,rvec x[],t_edsamyn *edyn);
/* this function has to be called before the (LINCS, SHAKE, etc. ) constraints are applied */

extern void init_edsam(FILE *log,t_topology *top,t_inputrec *ir,
		       t_mdatoms *md,int start,int homenr,t_commrec *cr,
		       t_edsamyn *edyn);
/* main init_routine - calls init_edi in a loop for every .edi-cycle contained in the input file, 
   creates a NULL terminated list of t_edpar structures */

extern void do_first_edsam(FILE *log,t_topology *top,
			   t_mdatoms *md,int start,int homenr,t_commrec *cr,
			   rvec x[],matrix box,t_edsamyn *edyn,bool bHaveConstr);

extern void do_edsam(FILE *log,t_topology *top,t_inputrec *ir,int step,
		     t_mdatoms *md,int start,int homenr,t_commrec *cr,
                     rvec x[],rvec xold[],rvec f[],matrix box,
                     t_edsamyn *edyn,bool bHave_force);
/* this function implements the edsam, constraints and the .edo monitoring functionality */

extern void finish_edsam(FILE *log,t_topology *top,t_inputrec *ir,
			 t_mdatoms *md,int start,int homenr,t_commrec *cr,
			 t_edsamyn *edyn);
/* after calling this routine the pointer to the data-structre t_edpar *edi is invalid, 
   all memory is deallocated. No further calls to any routine of this interface possible */


extern void ed_open(int nfile,t_filenm fnm[],t_edsamyn *edyn,t_commrec *cr);

int read_edi_file(t_edsamyn *edyn,t_edpar *edi,int nr_mdatoms);

int read_edi(FILE* in,t_edsamyn *edyn,t_edpar *edi,int nr_mdatoms,int edi_nr);

void init_edi(FILE *log,t_topology *top,t_inputrec *ir,
	      t_mdatoms *md,int start,int homenr,t_commrec *cr,
	      t_edsamyn *edyn,t_edpar *edi);
/* init-routine called for every *.edi-cycle, initialises t_edpar structure */

void check(char *line,char *label);

int read_checked_edint(FILE *file,char *label);
/* read the parameter from .edi file and check for its label --> fatal_error if wrong */

real read_checked_edreal(FILE *file,char *label);
/* read the parameter from .edi file and check for its label --> fatal_error if wrong */

int read_edint(FILE *file,bool *bEOF);

int read_edint2(FILE *file);

real read_edreal(FILE *file);

void read_edx(FILE *file,int number,int *anrs,rvec *x);

void read_edvecs(FILE *in,int nr,t_edvecs *vecs);
/* calls read_edvec for the vector groups, only for flooding there is an extra call */

void read_edvec(FILE *in,int nr,t_eigvec *tvec);

void scan_edvec(FILE *in,int nr,rvec *vec);
/*that is the (very private) scanf routine called by read_edvec */

int ed_constraints(t_edsamyn *edyn);
/* return value is 1 if constraints are switched on, 0 otherwise */

void put_in_origin(int nr,rvec *x,int nmass,int *masnrs,real *mass,real tmass);
/* puts center of mass of x[edi->fitnrs[i]] into com of reference structure */ 

void do_edfit(int natoms,rvec *xp,rvec *x,matrix R,t_edpar *edi); 
/* calculates rotation matrix R for least squares rotational fit */

void fitit(int nr,rvec *x,t_edpar *edi,rvec *transvec,matrix rmat);
/* fits x[edi->fitnrs[i]] bzw x[edi->masnrs[i]] which should be equivalent
   X has to contain ALL Atoms */

void rmrotfit(int ned,rvec *x,matrix rotmat);  
/* remove rotation */

void rmtransfit(int ned,rvec *x,rvec *transvec); 
/* remove fit */

void transfit(int ned,rvec *x,rvec *transvec); 
/* in fit direction */

void rotate_x(int ned,rvec *x,matrix rotmat); 
/* apply rotation of rotmat */

void correct_vel(int nr,rvec* vel,rvec *v1,rvec *v2,real fact,rvec* vcorr);

void correct_force(int nr,rvec* force,rvec *v1,rvec *v2,real fact,t_topology *top);

void remove_pbc_effect(int ned,rvec *transvec_compact,matrix box,t_edpar *edi);

void project(rvec *x,t_edpar *edi,char *mode);
/* wrapper: project vector x onto all edi->vecs (mon, linfix,...) */

void project_to_eigvectors(rvec *x,t_eigvec *vec,t_edpar *edi,char *mode);
/* project vector x, uses atoms sav.anrs[i] of x,
   if mode  is "x" it subtract average positions prior to projection
   and add them afterwards to retain the unchanged vector x
   mode = "x","v","f" -> store in xproj, vproj or fproj
   XXX mass-weighting is applied */

inline real projectx(t_edpar *edi,rvec *x,rvec *vec);
/* does not subtract average positions, projection on single eigenvector is returned
   used by: do_linfix, do_linacc, do_radfix, do_radacc, do_radcon
   here average position is subtracted in ed_cons prior to call to projectx */

inline real projectf(t_edpar *edi,rvec *x,rvec *vec);
/* same as projectx, but mass-weighting is applied differently -> forces  */

void rad_project(t_edpar *edi,rvec *x,t_eigvec *vec);
/* specialized:   projection is stored in vec->refproj
   ---> used for radacc, radfix, radcon  and center of flooding potential
   subtracts average positions, projects vector x, uses atoms sav.anrs[i] of x, */

real calc_radius(t_eigvec *vec);

void rmfit(int ned,rvec *x,rvec *transvec,matrix rotmat);

void rotate_vec(int nr,rvec *x,matrix rotmat);

void ed_cons(rvec *x,t_edpar *edi,int step);
/* applies all the following  constraints*/

void do_linfix(rvec *x,t_edpar *edi,int step);
void do_linacc(rvec *x,t_edpar *edi);
void do_radfix(rvec *x,t_edpar *edi,int step);
void do_radacc(rvec *x,t_edpar *edi);
void do_radcon(rvec *x,t_edpar *edi);

void write_edo(t_edpar *edi,int step,real rmsd);

void write_proj(FILE *out,t_edpar *edi,char *mode);

void do_write_proj(FILE *out,t_eigvec *vec,char *mode);

void write_edidx(FILE *out,t_edpar *edi);

void free_t_edvec(t_eigvec *vecs);

void free_t_edx(t_edx *edx);

void free_t_edlocals(t_edlocals *p);

void free_single_edpar(t_edpar *edi);

struct t_ed_local *init_local(void);

void free_local(struct t_ed_local *local);

static void free_t_fitit(struct t_fitit *);

static void free_t_do_edfit(struct t_do_edfit *);

static void free_t_remove_pbc_effect(struct t_remove_pbc_effect *);

static void free_t_do_edsam(struct t_do_edsam *);

static void free_t_do_radcon(struct t_do_radcon *);

real get_rmsd(t_edpar *edi,rvec *x);



#endif	/* _edsam_h */






