/*
 * $Id: repl_ex.h,v 1.5 2005/08/12 09:33:49 hess Exp $
 * 
 *                This source code is part of
 * 
 *                 G   R   O   M   A   C   S
 * 
 *          GROningen MAchine for Chemical Simulations
 * 
 *                        VERSION 3.2.0
 * Written by David van der Spoel, Erik Lindahl, Berk Hess, and others.
 * Copyright (c) 1991-2000, University of Groningen, The Netherlands.
 * Copyright (c) 2001-2004, The GROMACS development team,
 * check out http://www.gromacs.org for more information.

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * If you want to redistribute modifications, please consider that
 * scientific software is very special. Version control is crucial -
 * bugs must be traceable. We will be happy to consider code for
 * inclusion in the official distribution, but derived work must not
 * be called official GROMACS. Details are found in the README & COPYING
 * files - if they are missing, get the official version at www.gromacs.org.
 * 
 * To help us fund GROMACS development, we humbly ask that you cite
 * the papers on the package - you can find them in the top README file.
 * 
 * For more info, check our website at http://www.gromacs.org
 * 
 * And Hey:
 * Gallium Rubidium Oxygen Manganese Argon Carbon Silicon
 */
#ifndef _repl_ex_h
#define _repl_ex_h

#include "typedefs.h"

/** 
 ** New structs and functions of the Temperature Enhanced Essential dynamics
 ** Replica EXchange (TEE-REX) module are included here (-> least fiddling with 
 ** Makefiles & #include statements).
 ** Reference: M. B. Kubitzki, B. L. de Groot, Biophys J 92, 4262-4270 (2007)
 ** MK, 21.01.2008
 **/

/* replica exchange */
typedef struct {
  int  repl;
  int  nrepl;
  bool bNPT;
  real *temp;
  real *pres;
  int  *ind;
  int  nst;
  int  seed;
  int  nattempt[2];
  real *prob_sum;
  int  *nexchange;
} gmx_repl_ex_t;

/* TEE-REX */
typedef struct {
  bool       bTEEREX;        /* do TEE-REX sampling?                                                     */
  bool       bidGrpTemp;     /* TRUE if T_es == T_0 -> no T_es-coupl. for this repl, just GMX T-coupl.   */
  real       lambda_es;      /* Berendsen T_es-coupling parameter                                        */
  real       tau_es;         /* T_es coupling constant, input via TAU variable in sam.edi (make_edi -tau)*/
  real       *T_es;          /* essential subspace target temperatures                                   */
  real       *Tsys_eff;      /* effective system temperatures ONLY used in REMD exchange criterion       */
  real       nrdf_sys;       /* #DOF of the system                                                       */
  int        a, b;           /* replica indices used for {es}-scaling after exchange                     */
  t_commrec  *mcr;           /* pointer to mcr; needed for bypassing update(...,cr,...) -> gmx_sum(mcr)  */
  t_mdatoms  *mdatoms;       /* pointer to mdatoms; needed for T-scaling after TEE-REX exchange          */
  t_inputrec *ir;            /* pointer to ir; otherwise additional parameters in replica_exchange()     */
  t_groups   *grps;          /* dito.                                                                    */
} t_teerex;  


/*
 * replica exchange 
 */
extern gmx_repl_ex_t *init_replica_exchange(FILE *fplog,
					    const t_commrec *mcr,
					    const t_state *state,
					    const t_inputrec *ir,
					    int nst,int init_seed);

extern bool replica_exchange(FILE *fplog,const t_commrec *mcr,gmx_repl_ex_t *re,
			     t_teerex *teerex, /* TEE-REX add on */
                             t_edsamyn *edyn,  /* TEE-REX add on */
			     t_state *state,real epot,int step,real time);
/* Attempts replica exchange.
 * Returns TRUE if this state has been exchanged.
 */

extern void print_replica_exchange_statistics(FILE *fplog,gmx_repl_ex_t *re);





/**
 ** Temperature Enhanced Essential dynamics Replica EXchange (TEE-REX)
 **/
extern t_teerex *init_TEEREX(FILE *log,t_commrec *mcr,t_inputrec *ir,t_groups *grps,
			     t_mdatoms *mdatoms,t_edsamyn *edyn,gmx_repl_ex_t *re);

extern void TEEREX_tcoupl(FILE *log,int step,rvec xprime[],t_edsamyn *edyn,
			  gmx_repl_ex_t *re,t_state *state,t_teerex *teerex,real fac);

#endif 
